/*
 * gabien-app-ppj - Editor/Player for 'PiyoPiyo' music files
 * Written starting in 2015 by contributors (see CREDITS.txt)
 * To the extent possible under law, the author(s) have dedicated all copyright and related and neighboring rights to this software to the public domain worldwide. This software is distributed without any warranty.
 * You should have received a copy of the CC0 Public Domain Dedication along with this software. If not, see <http://creativecommons.org/publicdomain/zero/1.0/>.
 */
package libpiyo.export;

import gabien.uslx.audio.WavIO;
import libpiyo.PiyoPiyoFile;
import libpiyo.PiyoPiyoPlayer;
import libpiyo.SimpleMixer;

import java.io.IOException;
import java.io.OutputStream;

public class WavOut {
    public static void perform(OutputStream fos, PiyoPiyoFile coreFile, int startFrame, int ignoreFrame, int frameCount) throws IOException {
        final SimpleMixer sm = new SimpleMixer(128, true, true);
        final PiyoPiyoPlayer iPlayer = new PiyoPiyoPlayer(coreFile, sm);
        final int sampleRate = 48000;
        final double time = 1d / sampleRate;
        // Alter pitch. An explanation of this:
        // PiyoPiyoPlayer's timescale is just in seconds (you advance it and it updates the mixer channels).
        // However, the pitch values it calculates assumes a 22050hz "base frequency".
        // So the pitch has to be "virtually lowered" to account for this.
        iPlayer.pitchScale = 22050d / sampleRate;
        iPlayer.setCurrentFrame(startFrame);
        // Regarding frameCount: Some complex loops could fool a more flexible approach.
        int ignoreSampleCount = (int) (ignoreFrame * sampleRate * iPlayer.piyoFile.beatTime());
        final int sampleFrameCount = (int) (frameCount * sampleRate * iPlayer.piyoFile.beatTime());

        for (int i = 0; i < ignoreSampleCount; i += 32) {
            iPlayer.updateEnvelopes(time * 32);
            iPlayer.updateBeats(time * 32);
            // Stereo data.
            sm.pullData(Math.min(ignoreSampleCount - i, 32));
        }

        WavIO.writeWAV(fos, new WavIO.SourceS16() {
            int updateCycler = 0;
            int updateCycleLen = 4;
            short[] updateContents;

            @Override
            public int sampleRate() {
                return sampleRate;
            }

            @Override
            public int frameCount() {
                return sampleFrameCount;
            }

            @Override
            public int channels() {
                return 2;
            }

            @Override
            public void nextFrame(short[] buffer) {
                if (updateContents == null) {
                    iPlayer.updateEnvelopes(time * updateCycleLen);
                    iPlayer.updateBeats(time * updateCycleLen);
                    updateContents = sm.pullData(updateCycleLen);
                }

                for (int i = 0; i < buffer.length; i++)
                    buffer[i] = updateContents[updateCycler++];

                if (updateCycler == updateContents.length) {
                    updateCycler = 0;
                    updateContents = null;
                }
            }
        });
    }
}
