/*
 * gabien-app-ppj - Editor/Player for 'PiyoPiyo' music files
 * Written starting in 2015 by contributors (see CREDITS.txt)
 * To the extent possible under law, the author(s) have dedicated all copyright and related and neighboring rights to this software to the public domain worldwide. This software is distributed without any warranty.
 * You should have received a copy of the CC0 Public Domain Dedication along with this software. If not, see <http://creativecommons.org/publicdomain/zero/1.0/>.
 */
package piyopiyoj.games.edittools;

import java.io.OutputStream;

import gabien.GaBIEn;
import gabien.ui.UIElement;
import gabien.ui.UILabel;
import gabien.ui.UIPublicPanel;
import gabien.ui.UISplitterLayout;
import gabien.ui.UITextButton;
import gabien.uslx.append.IConsumer;
import libpiyo.export.MidOut;
import libpiyo.export.WavOut;
import libpiyo.export.XMOut;
import piyopiyoj.games.MidiParameterGame;
import piyopiyoj.games.NotesheetGame;

/**
 * 'tool' for export format panel.
 * Created on 24th May 2022.
 */
public class ExportEditTool extends DefaultSemanticsTool {

    @Override
    public UIElement toolOpened(final NotesheetGame editor) {
        UITextButton exportFile = new UITextButton("Intro", 16, new Runnable() {
            @Override
            public void run() {
                editor.root.fileDialog(true, new IConsumer<String>() {
                    @Override
                    public void accept(String s) {
                        if (s == null)
                            return;
                        try {
                            OutputStream fos = GaBIEn.getOutFile(s);
                            WavOut.perform(fos, editor.root.coreFile, 0, 0, editor.root.coreFile.loopEnd);
                            fos.close();
                        } catch (Exception e) {
                            e.printStackTrace();
                            editor.root.switchError(e, editor);
                        }
                    }
                }, "Export WAV");
            }
        });
        UITextButton exportLoopFile = new UITextButton("Loop", 16, new Runnable() {
            @Override
            public void run() {
                editor.root.fileDialog(true, new IConsumer<String>() {
                    @Override
                    public void accept(String s) {
                        if (s == null)
                            return;
                        try {
                            OutputStream fos = GaBIEn.getOutFile(s);
                            // Creates a perfect loop.
                            WavOut.perform(fos, editor.root.coreFile, 0, editor.root.coreFile.loopEnd, editor.root.coreFile.loopEnd - editor.root.coreFile.loopStart);
                            fos.close();
                        } catch (Exception e) {
                            e.printStackTrace();
                            editor.root.switchError(e, editor);
                        }
                    }
                }, "Export WAV");
            }
        });
        UITextButton exportMidiFile = new UITextButton("MIDI (*.mid)", 16, new Runnable() {
            @Override
            public void run() {
                editor.root.switcher(new MidiParameterGame(editor.root, new Runnable() {
                    @Override
                    public void run() {
                        editor.root.fileDialog(true, new IConsumer<String>() {
                            @Override
                            public void accept(String s) {
                                editor.root.switcher(editor);
                                if (s == null)
                                    return;
                                try {
                                    OutputStream fos = GaBIEn.getOutFile(s);
                                    MidOut.perform(fos, editor.root.coreFile);
                                    fos.close();
                                } catch (Exception e) {
                                    e.printStackTrace();
                                    editor.root.switchError(e, editor);
                                }
                            }
                        }, "Export MID");
                    }
                }));
            }
        });
        UITextButton exportXMFile = new UITextButton("FastTracker II / MilkyTracker (*.xm)", 16, new Runnable() {
            @Override
            public void run() {
                editor.root.fileDialog(true, new IConsumer<String>() {
                    @Override
                    public void accept(String s) {
                        if (s == null)
                            return;
                        try {
                            OutputStream fos = GaBIEn.getOutFile(s);
                            XMOut.perform(fos, editor.root.coreFile, GaBIEn.nameOf(editor.root.rememberFileName), 32);
                            fos.close();
                        } catch (Exception e) {
                            e.printStackTrace();
                            editor.root.switchError(e, editor);
                        }
                    }
                }, "Export XM");
            }
        });
        UIElement rowWAV = UISplitterLayout.produceSideAlignedList(false, false, new UILabel("Wave (*.wav): ", 16), exportFile, new UIPublicPanel(4, 16), exportLoopFile, new UIPublicPanel(1, 1));
        UIElement rowTrk = UISplitterLayout.produceSideAlignedList(false, false, new UILabel("Sequenced: ", 16), exportMidiFile, new UIPublicPanel(16, 16), exportXMFile, new UIPublicPanel(1, 1));

        return NoteEditTool.generateBack(UISplitterLayout.produceSideAlignedList(false, true,
                rowWAV,
                rowTrk,
                new UIPublicPanel(1, 1)
        ), editor);
    }

}
